(*
  CPF-UDF 1.0 - Funes UDF para formatao e validao de CPF.
  Compatvel com InterBase e FireBird.

  Autor....: Daniel Pereira Guimares
  E-mail...: tecnobyte@ulbrajp.com.br
  Home-Page: www.ulbrajp.com.br/~tecnobyte

  Esta biblioteca  Open-Source!
*)

library cpf_udf;

uses
  SysUtils;

{ *** Rotinas internas *** }

function Digitos(const S: string): string;
var
  I: integer;
begin
  Result := '';
  for I := 1 to Length(S) do
    if S[I] in ['0'..'9'] then
      Result := Result + S[I];
end;

function ValidarCPF(S: string): boolean;
var
  CPF: array[1..11] of byte;
  I, Soma, Dig1, Dig2: integer;
  Ok: boolean;
  Temp: byte;
begin
  Result := false;

  S := Digitos(S);

  { Se no possui 11 dgitos ento no  CPF vlido }
  if Length(S) <> 11 then
    Exit;

  for I := 1 to 11 do
    CPF[I] := StrToInt(S[I]);

  { Se todos os 11 dgitos so iguais ento (provavelmente)
    no  CPF vlido. }
  Ok := false;
  Temp := CPF[1];
  for I := 2 to 11 do
    if CPF[I] <> Temp then begin
      Ok := true;
      Break;
    end;

  if not Ok then { Todos os dgitos so iguais, ento sai }
    Exit;

  { Clculo do 1 dgito }
  Soma := 0;
  for I := 1 to 9 do
    Soma := Soma + CPF[I] * (11 - I);

  Dig1 := 11 - (Soma mod 11);
  if Dig1 > 9 then
    Dig1 := 0;

  { Clculo do 2 dgito }
  Soma := 0;
  for I := 1 to 10 do
    Soma := Soma + CPF[I] * (12 - I);

  Dig2 := 11 - (Soma mod 11);
  if Dig2 > 9 then
    Dig2 := 0;

  Result := (Dig1 = CPF[10]) and (Dig2 = CPF[11]);
end;

{ *** UDFs *** }

function cpf_Limpar(Str: PChar): PChar; cdecl; export;
begin
  Result := Str;
  StrPCopy(Str, Digitos(Str));
end;

function cpf_Formatar(Str: PChar): PChar; cdecl; export;
var
  S: string;
begin
  Result := Str;

  S := Digitos(Str);

  if Length(S) = 11 then
    StrPCopy(Str, Copy(S, 1, 3) + '.' + Copy(S, 4, 3) + '.' +
      Copy(S, 7, 3) + '-' + Copy(S, 10, 2));
end;

function cpf_Validar(Str: PChar): SmallInt; cdecl; export;
begin
  Result := SmallInt(ValidarCPF(Str));
end;

function cpf_ValidarNum(var X: double): SmallInt; cdecl; export;
begin
  Result := SmallInt(ValidarCPF(FloatToStr(Int(X))));
end;

exports
  cpf_Limpar,
  cpf_Formatar,
  cpf_Validar,
  cpf_ValidarNum;
  
begin
end.
